@@ -1,4 +1,4 @@
-/* $Id: milter-greylist.c,v 1.230 2010/04/18 04:03:56 manu Exp $ */
+/* $Id: milter-greylist.c,v 1.235 2010/07/12 01:38:14 manu Exp $ */
 
 /*
  * Copyright (c) 2004-2007 Emmanuel Dreyfus
@@ -34,7 +34,7 @@
 #ifdef HAVE_SYS_CDEFS_H
 #include <sys/cdefs.h>
 #ifdef __RCSID  
-__RCSID("$Id: milter-greylist.c,v 1.230 2010/04/18 04:03:56 manu Exp $");
+__RCSID("$Id: milter-greylist.c,v 1.235 2010/07/12 01:38:14 manu Exp $");
 #endif
 #endif
 
@@ -832,6 +832,7 @@ real_header(ctx, name, value)
 	strcat(h->h_line, sep);
 	strcat(h->h_line, value);
 	strcat(h->h_line, crlf);
+	h->h_len = len;
 
 	TAILQ_INSERT_TAIL(&priv->priv_header, h, h_list);
 
@@ -912,6 +913,7 @@ real_body(ctx, chunk, size)
 			exit(EX_OSERR);
 		}
 
+		b->b_len = strlen(crlf);
 		TAILQ_INSERT_TAIL(&priv->priv_body, b, b_list);
 
 		priv->priv_msgcount += strlen(crlf);
@@ -945,6 +947,7 @@ real_body(ctx, chunk, size)
 
 		memcpy(b->b_lines + priv->priv_buflen, chunk, i);
 		b->b_lines[linelen] = '\0';
+		b->b_len = linelen;
 		priv->priv_buflen = 0;
 
 		TAILQ_INSERT_TAIL(&priv->priv_body, b, b_list);
@@ -1002,6 +1005,7 @@ real_eom(ctx)
 		}
 
 		b->b_lines = priv->priv_buf;
+		b->b_len = priv->priv_buflen - 1;
 		b->b_lines[priv->priv_buflen - 1] = '\0';
 
 		priv->priv_buf = NULL;
@@ -1242,6 +1246,8 @@ passed:
 			/* Silently ignore other codes, just report ACL */	
 			if (priv->priv_last_whitelist != 0) {
 				priv->priv_last_whitelist = 0;
+				if (whystr[0] != '\0')
+					mystrlcat (whystr, ", ", HDRLEN);
 				mystrlcat (whystr, "ACL %a matched", HDRLEN);
 			}
 
@@ -2651,6 +2657,22 @@ fstring_expand(priv, rcpt, fstring)
 			break;
 		}
 
+		case 'H': { /* SpamAssassin information */
+#ifdef USE_SPAMD
+			fstr_len = 2;
+			switch(*(ptok + 1)) {
+			case 's': { /* score */
+				char buf[QSTRLEN + 1];
+				(void)snprintf(buf, sizeof(buf), "%g",
+				    (double)priv->priv_spamd_score10/10);
+				mystrncat(&outstr, buf, &outmaxlen);
+				break;
+				}
+			}
+#endif /* USE_SPAMD */
+			break;
+		}
+
 		case 'I': {	/* Sender machine / cidr, eg: %I{/24} */
 			char ipstr[IPADDRSTRLEN + 1];
 			struct sockaddr_storage addr;
