@@ -1,7 +1,7 @@
-/* $Id: milter-greylist.h,v 1.80 2009/11/01 02:03:33 manu Exp $ */
+/* $Id: milter-greylist.h,v 1.84 2010/07/12 01:38:14 manu Exp $ */
 
 /*
- * Copyright (c) 2004-2007 Emmanuel Dreyfus
+ * Copyright (c) 2004-2010 Emmanuel Dreyfus
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -180,11 +180,13 @@ struct rcpt {
 
 struct header {
 	char *h_line;
+	size_t h_len;
 	TAILQ_ENTRY(header) h_list;
 };
 
 struct body {
 	char *b_lines;
+	size_t b_len;
 	TAILQ_ENTRY(body) b_list;
 };
 
@@ -370,5 +372,21 @@ size_t mystrlcat(char *, const char *src, size_t size);
  */
 /* #define WORKAROUND_LIBMILTER_RACE_CONDITION */
 
-#endif /* _MILTER_GREYLIST_H_ */
+/*
+ * We may want to check SOCK_CLOEXEC as well for close-on-exec
+ * availlability for sockets. On the other hand, if the 
+ * functionnality is not there, fcntl(F_SETFD) will just 
+ * silently fail.
+ */
+#ifdef FD_CLOEXEC
+#define SET_CLOEXEC(fd)  do {			   		\
+	int flags = fcntl((fd), F_GETFD, 0);		    	\
+								\
+	if (flags != -1)					\
+		(void)fcntl(fd, F_SETFD, flags|FD_CLOEXEC);	\
+} while (0 /* CONSTCOND */)
+#else /* FD_CLOEXEC */
+#define SET_CLOEXEC(fd)
+#endif /* FD_CLOEXEC */
 
+#endif /* _MILTER_GREYLIST_H_ */
